/*
 * DigitExtraction.java
 *
 * Created on October 6, 2004, 12:36 PM
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.io.File;

/**
 *
 * @author  xiao
 */
public class DigitExtraction {
    
    final int MAXIMAGES = 56;
    private String sept = String.valueOf(System.getProperty("file.separator").toCharArray()[0]);
    
    private int[][] bwImage;
    private int[] digResult;
    private int[] guessResult;
    private int maxrows;
    private int maxcols;
    private Vector [] regionx;
    private Vector [] regiony;
    private int imageNum=0;      //Count how many digit's images in a graph
    private FeatureExtraction [] digits;
    private DTClassifier myDT;
    private NNClassifier myNN;
    
    private boolean print_feature = false;
    
    /** Creates a new instance of DigitExtraction
     *   input is a bw image mapping in a 2D array
     */
    public DigitExtraction(int[][] inputImage, boolean print) {
        bwImage = inputImage;
        print_feature = print;
        maxrows = bwImage.length;
        maxcols = bwImage[0].length;
        
        System.out.println("Extraction Rows: " + maxrows + " Cols: " + maxcols);
        
        regionx = new Vector [MAXIMAGES];  //x-value of region
        regiony = new Vector [MAXIMAGES];  //y-value of region
        digits = new FeatureExtraction [MAXIMAGES];
    }
    
    
    /**Find the connected region of the image by recursion
     * Note that we limited the number of digits from one single image to
     * the MAXIMAGES to avoid stack overflow problem.
     */
    public void regionFinder(int startrow, int startcol) {
        for (int rows=startrow; rows<maxrows; rows++) {
            for (int cols=startcol; cols<maxcols; cols++) {
                if (bwImage[rows][cols]== 0) {  /* black pixel in the image*/
                    
                    regionx[imageNum] = new Vector();
                    regiony[imageNum] = new Vector();
                    
                    savePixel(rows, cols);  //recursively find neighbor pixels
                    
                    imageNum++;
                    
                    if (imageNum >= MAXIMAGES) {
                        System.out.println("Found more possible regions in the image, but only " + MAXIMAGES + " allowed!");
                        return;
                    }
                }
            }
        }
        System.out.println("Total # of regions found: " + imageNum);
        
    }
    
    public void regionExtration() {
        for (int i = 0; i < imageNum; i++) {
            digits[i] = new FeatureExtraction(regionx[i], regiony[i], print_feature);
        }
        return;
    }
    
    /**Save the black pixels to region Vector.
     */
    private void savePixel(int rows, int cols) {
        if (bwImage[rows][cols] == 0) { /*still a black pixel*/
            regionx[imageNum].add(new Integer(cols));
            regiony[imageNum].add(new Integer(rows));
            
            bwImage[rows][cols] = 0xFFFFFF;  // make the pixel white
            findNeighbor(rows, cols);
        }
        return;
    }
    
    /**Naive way of finding a region recusively. Searhing for the 8 neighbor of
     * the current pixel. Demonstrated below as "#".
     * |#|#|#|
     * |#|o|#|
     * |#|#|#|
     */
    private void findNeighbor(int rows, int cols) {
        if (!isEdge(rows, cols)) { /*avoid to check the pixels on the edges*/
            savePixel(rows-1, cols-1);
            savePixel(rows-1, cols);
            savePixel(rows-1, cols+1);
            savePixel(rows, cols-1);
            savePixel(rows, cols+1);
            savePixel(rows+1, cols-1);
            savePixel(rows+1, cols);
            savePixel(rows+1, cols+1);
        }
        return;
    }
    
    /**Check whether a pixel is on an edge
     */
    private boolean isEdge(int rows, int cols) {
        return ((rows == 0 || rows == (maxrows-1)) || (cols == 0 || cols == (maxcols-1)));
    }
    
    /** save the feature set for the image*/
    public void saveFeatureText(String filename) {
        
        try {
            File saveFile = new File(filename);
            OutputStream output = new BufferedOutputStream(new FileOutputStream(saveFile));
            for (int i=0; i<imageNum; i++) {
                output.write(digits[i].toString().getBytes());
            }
            output.close();
        }
        catch(IOException e) {
            JOptionPane.showMessageDialog((Component)null, "Error while saving the features.",
            "File Save Warning", JOptionPane.WARNING_MESSAGE);
            return;
        }
    }
    
    public String toString() {
        String s = " Total # of digits read: " + imageNum;
        
        for (int i = 0; i < imageNum; i++) {
            s += digits[i].toString();
        }
        return s;
    }
    
    private int[] getBounds(int i) {
        int min_x = Integer.MAX_VALUE,
        min_y = Integer.MAX_VALUE,
        max_x = Integer.MIN_VALUE,
        max_y = Integer.MIN_VALUE,
        temp = -1;
        
        int[] bbox = new int[4];
        
        for (int j = 0; j < regionx[i].size(); j++) {
            temp = ((Integer)regionx[i].get(j)).intValue();
            if (temp < min_x) {
                min_x = temp;
            }
            if (temp > max_x) {
                max_x = temp;
            }
            temp = ((Integer)regiony[i].get(j)).intValue();
            if (temp < min_y) {
                min_y = temp;
            }
            if (temp > max_y) {
                max_y = temp;
            }
        }
        
        bbox[0] = min_x;
        bbox[1] = max_x;
        bbox[2] = min_y;
        bbox[3] = max_y;
        
        return bbox;
    }
    
    public int[] getBoundBoxes() {
        int[] bbarray = new int[4*imageNum];
        int[] sbox = new int[4];
        
        for (int i = 0; i < imageNum; i ++) {
            sbox = getBounds(i);
            for (int j = 0; j < 4; j++) {
                bbarray[4*i+j] = sbox[j]; // pack the box co-ordinates into the array
            }
        }
        return bbarray;
    }
    
    public int[] DTclassify() {
        digResult = new int[imageNum];
        guessResult = new int[imageNum];
        myDT = new DTClassifier();
        for (int i = 0; i < imageNum; i++) {
            myDT.setDigInfo(digits[i].toFeature());
            digResult[i] = myDT.classifyDT();
            guessResult[i] = myDT.getGuess();
        }
        
        return digResult;
    }
    
    public int[] NNclassify() {
        
        String current_dir = "";
        JFileChooser chooser = new JFileChooser();
        //chooser.setCurrentDirectory(new File(current_dir));
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setDialogTitle("Open Directory Dialog");
        
        try {
            int result = chooser.showOpenDialog(null);
            
            if (result == JFileChooser.APPROVE_OPTION) {
                File f = chooser.getSelectedFile();
                current_dir = f.getAbsolutePath();
                System.out.println(current_dir);
                
                digResult = new int[imageNum];
                guessResult = new int[imageNum];
                myNN = new NNClassifier(current_dir + sept);
                for (int i = 0; i < imageNum; i++) {
                    myNN.setDigMap(digits[i].getDigMap());
                    digResult[i] = myNN.classifyNN();
                    guessResult[i] = myNN.getGuess();
                }
            }
        }
        catch (Exception ex) {
            JOptionPane.showMessageDialog((Component)null,
            "Error while trying to open:" + ex.toString(),
            "Directory Open Warning", JOptionPane.WARNING_MESSAGE);
        }
        return digResult;
    }
    
    public int[] getGuessResult() {
        return guessResult;
    }
    
    public void saveLib(int num) {
        int i, j, k;
        String out="";
        
        for (i = 0; i < imageNum; i++) {
            int[][] p = digits[i].getDigMap();
            for (j = 0; j < p.length; j++) {
                for (k = 0; k < p[0].length; k++) {
                    out += p[j][k];
                    out += " ";
                }
                out += "\n";
            }
            out += "#\n";
        }
        
        String current_dir = null;
        JFileChooser chooser = new JFileChooser();
        //chooser.setCurrentDirectory(new File(current_dir));
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setDialogTitle("Directory Selection Dialog");
        
        try {
            int result = chooser.showOpenDialog(null);
            
            if (result == JFileChooser.APPROVE_OPTION) {
                File f = chooser.getSelectedFile();
                current_dir = f.getAbsolutePath();
                
                System.out.println(current_dir);
                File saveFile = new File(current_dir + sept + num + "_lib.dat");
                OutputStream output = new BufferedOutputStream(new FileOutputStream(saveFile));
                output.write(out.getBytes());
                output.close();               
            }
        }
        catch(IOException e) {
            JOptionPane.showMessageDialog((Component)null, "Error while saving the library.",
            "File Save Warning", JOptionPane.WARNING_MESSAGE);
            return;
        }
    }
}
