/*
 * FeatureExtraction.java
 *
 * Created on October 6, 2004, 12:36 PM
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.io.File;
import java.awt.image.BufferedImage;
import java.awt.image.*;
/**
 *
 * @author  xiao
 */
public class FeatureExtraction {
    
    private int maxX = 0;    // bounding box size and co-ordinates
    private int maxY = 0;
    private int minX = Integer.MAX_VALUE;
    private int minY = Integer.MAX_VALUE;
    private int height = 0;  // height of region
    private int width = 0;   // width of region
    
    private int hlongY = 0;  //number of vertical lines longer than vlineRatio*height of box
    private int hlongX = 0;  //number of horizental lines longer than hlineRatio*width of box
    private double hlineRatio = 0.9;  //Threshold to check hlongX (long horizental line) number
    private double vlineRatio = 0.9;  //Threshold to check hlongY (long verticall line) number
    
    private int holes = 0;   //number of holes. 4-neighborhood to check hole, coz using 8-neighbor to check region!
    private int upholes = 0; //number of holes on top part of region
    private int downholes = 0; //number of holes at the bottom part of region
    private int centerholes = 0;  //number of holes in center
    //upholes+downholes+centerholes = holes
    private double topPercent = 0.4;  //Must be less than 0.5
    //top part of region for a hole is defined as the center of the hole's height is higher
    //  than (ceil)topPercent*height. Similiar value to downholes. The other holes is center hole
    private int holet = Integer.MAX_VALUE;
    private int holeb = Integer.MIN_VALUE; //mark bottom/top of the hole
    private boolean uph=false;
    private boolean dnh=false; //check different holes
    private int notchs = 0;  //number of notchs
    
    private int area = 0;    // number of black pixels in the region
    private int boxArea = 0;  // number of pixels in the region
    private int holeArea = 0;  //area of the hole
    private int notchArea = 0; //area of notch
    
    private double abRatio = 0.0;  // area/boxArea
    private double bbRatio = 0.0;  // Width/Height of box
    private double hbRatio = 0.0;  // holeArea/boxArea
    private double nbRatio = 0.0;  //notchArea/boxArea
    
    private Vector  regionx;  // x-values of region
    private Vector  regiony;  // y-values of region
    
    private int[][] pixels;    //store the pixel value of the box, used when finding the holes
    private boolean ahole = true;   //check hole
    private int wArea = 0;  //temp store the hole/notch area
    private int isHor = -1;   //mark pixel as -1 when is horizental line. Must be a negative integer!
    private int isVer = -2;   //mark the vertical line as -2. Be ready to compare with isHor, like isHor+isVer for some pixels
    private int isBoublePixels = 1; //How many minimum overlapped pixels to determine a parallel double-line
    
    /* set the print out function to true to see proceduces*/
    private boolean print_out = false;
    private JFrame disWindow;
    
    /** Creates a new instance of FeatureExtraction */
    public FeatureExtraction(Vector regionx, Vector regiony, boolean print) {
        
        this.regionx = regionx;
        this.regiony = regiony;
        
        print_out = print;
        /*get the features when construction the object*/
        getBound();
        getHole();
        if (print_out) {
            System.out.println("Printing out featured image ...");
            PrintImage(pixels);
        }
        getLongLine();
    }
    
    /**Calculate region/bound's info, like area, maxX, a(b)bRatio, etc.
     */
    private void getBound() {
        
        area = regionx.size();   /*number of black pixels in the region*/
        
        for (int i = 0; i < regionx.size(); i++) {
            int tempx = ((Integer)regionx.get(i)).intValue();
            int tempy = ((Integer)regiony.get(i)).intValue();
            if (tempx > maxX) maxX = tempx;
            if (tempx < minX) minX = tempx;
            if (tempy > maxY) maxY = tempy;
            if (tempy < minY) minY = tempy;
        }
        
        //System.out.println("min X: " + minX + "min Y: " + minY);
        //System.out.println("max X: " + maxX + "max Y: " + maxY);
        
        width = (maxX - minX + 1);  //image width/height
        height = (maxY - minY + 1);
        
        boxArea = width * height; /* number of pixels in the region*/
        if (boxArea <=0) System.out.println("Error! Box area is "+boxArea);
        
        abRatio = (double)area/boxArea;
        bbRatio = (double)width/height;
    }
    
    /**Cal number of holes/notchs in region
     * After check of the hole, pixels[][] value are all reset to 0
     */
    private void getHole() {
        getPixels();  // get the region map
        for (int rows = 0; rows < height; rows++) {
            for (int cols = 0; cols < width; cols++) {
                if (pixels[rows][cols] == 0xFFFFFF) {
                    ahole = true;
                    wArea=0;
                    uph=false;
                    dnh=false;
                    
                    holet = Integer.MAX_VALUE;
                    holeb = Integer.MIN_VALUE;
                    checkHole(rows, cols);
                    if (ahole) {
                        holes++;  //hole there
                        holeArea += wArea;  //cal hole area
                        holeType();  //check the type of the hole
                        if (uph) upholes ++;
                        else if (dnh) downholes ++;
                        else  centerholes++;
                    } else {
                        notchs++;  //must be a notch
                        notchArea += wArea;
                    }
                }
            }
        }
        hbRatio = (double)holeArea/boxArea;
        nbRatio = (double)notchArea/boxArea;
    }
    
    /**Check whether there is a hole in region when spreading from pixel (rows, cols)
     * There will be a hole if none of the 4-white neighbor hit the edge of the bounding box
     * If there is a hole, param ahole will be true.
     */
    private void checkHole(int rows, int cols) {
        if (pixels[rows][cols] == 0xFFFFFF) {
            wArea++;  //Increment hole/notch area
            if (rows < holet) holet=rows;
            if (rows > holeb) holeb=rows;  //check top/bottom of the hole
        }
        if (!isEdge(rows, cols)) {
            pixels[rows][cols] = 0x99CCFF;  //erase this white pixel
            goNeighbor(rows, cols);         //spread checking to neighbor
        } else {
            pixels[rows][cols] = 0xFFCC66; //marked edge notch pixels as 0xFFCC66
            ahole = false;                 //not a hole
            holet = Integer.MAX_VALUE;
            holeb = Integer.MIN_VALUE;
            goNotchEdge(rows, cols);
        }
        return;
    }
    
    /**Checking four neighborhood pixels for possible holes
     * Only 4-neighbor can be used here as demonstrated blow using "#"
     * | |#| |
     * |#|o|#|
     * | |#| |
     */
    private void goNeighbor(int rows, int cols) {
        if (!isEdge(rows, cols)) {
            if (pixels[rows-1][cols] == 0xFFFFFF)
                checkHole(rows-1, cols); 
            if (pixels[rows+1][cols] == 0xFFFFFF)
                checkHole(rows+1, cols);   
            if (pixels[rows][cols-1] == 0xFFFFFF)
                checkHole(rows, cols-1);
            if (pixels[rows][cols+1] == 0xFFFFFF)
                checkHole(rows, cols+1);
        }
        return;
    }
    
    /**Check the neighbor of a notch for the edge pixels
     * Black it if it's a white pixel
     */
    private void goNotchEdge(int rows, int cols){
        if (rows == 0 || rows == (height-1)) {  //top/bottom
            if (cols > 0 && (pixels[rows][cols-1]==0x99CCFF || pixels[rows][cols-1]==0xFFFFFF)) 
                checkHole(rows, cols-1);    //search edge
            if (cols < (width-1) && (pixels[rows][cols+1]==0x99CCFF || pixels[rows][cols+1]==0xFFFFFF)) 
                checkHole(rows, cols+1);
            if ((cols == 0 || cols == (width-1)) && rows == 0 && 
                            (pixels[rows+1][cols]==0x99CCFF || pixels[rows+1][cols]==0xFFFFFF)) 
                checkHole(rows+1, cols);    //top corner
            if ((cols == 0 || cols == (width-1)) && rows == (height-1) && 
                            (pixels[rows-1][cols]==0x99CCFF || pixels[rows-1][cols]==0xFFFFFF)) 
                checkHole(rows-1,cols);     //bottom corner
            if (rows == 0 && (pixels[rows+1][cols]==0x99CCFF || pixels[rows+1][cols]==0xFFFFFF)) 
                checkHole(rows+1, cols);   //search inside
            if (rows == (height-1) && (pixels[rows-1][cols]==0x99CCFF || pixels[rows-1][cols]==0xFFFFFF)) 
                checkHole(rows-1, cols);
        } else {        //left/right and no corner
            if (rows > 0 && (pixels[rows-1][cols]==0x99CCFF || pixels[rows-1][cols]==0xFFFFFF)) 
                checkHole(rows-1, cols);  //serach edge
            if (rows < (height-1) && (pixels[rows+1][cols]==0x99CCFF || pixels[rows+1][cols]==0xFFFFFF)) 
                checkHole(rows+1, cols);
            if (cols == 0 && (pixels[rows][cols+1]==0x99CCFF || pixels[rows][cols+1]==0xFFFFFF)) 
                checkHole(rows, cols+1);  //serach inside
            if (cols == (width-1) && (pixels[rows][cols-1]==0x99CCFF || pixels[rows][cols-1]==0xFFFFFF)) 
                checkHole(rows, cols-1);
        }
    }
    
    /**Check what's the type (upper/bottom/center) of the hole
     */
    private void holeType() {
        int centerW = (holet+holeb)/2;
        //System.out.println("Holes: " + holet + ", " + holeb);
        if (centerW > height -1 || centerW < 1) System.out.println("Hole Cutoff.");
        if (centerW <= height*topPercent) uph=true;  //top hole found
        else if (centerW >= (height-1-height*topPercent)) dnh=true;   //bottom hole
    }
    
    /**Calculate the number of lines longer than hlineRatio*width or vlineRatio*height of the bounding box
     */
    private void getLongLine() {
        getPixels();  //Get the pixel map of the region
        hlongX=0;
        hlongY=0;
        for (int rows = 0; rows < height; rows++) {
            for (int cols = 0; cols < width; cols++) {
                if (pixels[rows][cols] == 0) { /* black pixel*/
                    checkHLine(rows, cols);
                    checkVLine(rows, cols);
                }
            }
        }
    }
    
    /**Recover the pixel map of the region
     */
    private void getPixels() {
        /* initial a white map image*/
        pixels = new int [height][width];
        for (int rows = 0; rows < height; rows++) {
            for (int cols = 0; cols < width; cols++) {
                pixels[rows][cols] = 0xFFFFFF;
                
            }
        }
        /* put the black pixels in the map*/
        int tempx = 0, tempy = 0;
        for (int i = 0; i < regionx.size(); i++) {
            tempx = ((Integer)regionx.get(i)).intValue();
            tempy = ((Integer)regiony.get(i)).intValue();
            
            //System.out.println((tempx-minX) + " " + (tempy-minY));
            
            /* set the pixel to black*/
            pixels[tempy-minY][tempx-minX] = 0;
        }
    }
    
    /**Check whether there is a horizental line longer than hlineRatio*width
     */
    private void checkHLine(int rows, int cols) {
        if (countLine(rows, cols, isHor) >= Math.floor(hlineRatio*width)) {
            if (markLine(rows, cols, isHor))  hlongX++;
        }
    }
    
    /**Check whether there is a horizental line longer than hlineRatio*width
     */
    private void checkVLine(int rows, int cols) {
        if (countLine(rows, cols, isVer) >= Math.floor(vlineRatio*height)) {
            if (markLine(rows, cols, isVer))  hlongY++;
        }
    }
    
    /**Count how many pixels are in a line.
     */
    private int countLine(int rows, int cols, int lineDir) {
        int lineLength = 0;
        if (lineDir == isHor && pixels[rows][cols] != isHor && pixels[rows][cols] !=(isHor+isVer)) {  //not picked out yet
            for (int i = cols; i >= 0; i--) {
                if (pixels[rows][i] == 0 || pixels[rows][i] == isVer) {
                    lineLength++;
                } else {
                    break;
                }
            }
            for (int i = cols; i < width; i++) {
                if (pixels[rows][i] == 0 || pixels[rows][i] == isVer) {
                    lineLength++;
                } else {
                    break;
                }
            }
            lineLength--;  //remove the double count for [rows][cols]
        } else if (lineDir == isVer && pixels[rows][cols] != isVer && pixels[rows][cols] != (isHor+isVer)) {
            for (int i = rows; i >= 0; i--) {
                if (pixels[i][cols] == 0 || pixels[i][cols] == isHor) {
                    lineLength++;
                } else {
                    break;
                }
            }
            for (int i = rows; i < height; i++) {
                if (pixels[i][cols] == 0 || pixels[i][cols] == isHor) {
                    lineLength++;
                } else {
                    break;
                }
            }
            lineLength--;  //remove the double count for [rows][cols]
        } else {
        }
        return lineLength;
    }
    
    
    /**Mark the pixels of a line. = isHor for Horizental; = isVer for vertical; = isHor+isVer for both
     *The false lines are also marked as lineDir.
     */
    private boolean markLine(int rows, int cols, int lineDir) {
        int countDouble = 0;  //count the double points
        if (lineDir == isHor) {
            for (int i = cols; i >= 0; i--) {
                if (pixels[rows][i] == 0 || pixels[rows][i] == isVer) {
                    
                    pixels[rows][i] += isHor;  //mark as a horizontal line pixel
                    /* donot want to count adjacent lines separately
                    /* mark adjacent lines as horzontal */
                    if (rows > 0 && (pixels[rows-1][i] == isHor || pixels[rows-1][i] == (isHor+isVer))) {
                        countDouble++;
                    }
                    else if (rows > 0) {
                        pixels[rows-1][i] += isHor;
                    }
                    if (rows<(height-1) && (pixels[rows+1][i] == isHor || pixels[rows+1][i] == (isHor+isVer))){
                        countDouble++;
                    }
                    else if (rows < height-1) {
                        pixels[rows+1][i] += isHor;
                    }
                    
                }
            }
            for (int i = cols; i < width; i++) {
                if (pixels[rows][i] == 0 || pixels[rows][i] == isVer) {
                    if (i>cols) {
                        pixels[rows][i] += isHor;
                        if (rows>0 && (pixels[rows-1][i] == isHor || pixels[rows-1][i] == (isHor+isVer))) {
                            countDouble++;
                        }
                        else if (rows > 0) {
                            pixels[rows-1][i] += isHor;
                        }
                        if (rows<(height-1) && (pixels[rows+1][i] == isHor || pixels[rows+1][i] == (isHor+isVer))) {
                            countDouble++;
                        }
                        else if (rows < height-1) {
                            pixels[rows+1][i] += isHor;
                        }
                    }
                }
            }
        } else if (lineDir == isVer) {
            for (int i = rows; i >= 0; i--) {
                if (pixels[i][cols] == 0 || pixels[i][cols] == isHor) {
                    pixels[i][cols] += isVer;  //mark as a vertical line
                    if (cols>0 && (pixels[i][cols-1] == isVer || pixels[i][cols-1] == (isHor+isVer))) {
                        countDouble++;
                    }
                    else if (cols > 0) {
                        pixels[i][cols-1] += isVer;
                    }
                    if (cols<(width-1) && (pixels[i][cols+1] == isVer || pixels[i][cols+1] == (isHor+isVer))) {
                        countDouble++;
                    }
                    else if (cols < width-1) {
                        pixels[i][cols+1] += isVer;
                    }
                }
            }
            for (int i = rows; i < height; i++) {
                if (pixels[i][cols] == 0 || pixels[i][cols] == isHor) {
                    if (i > rows) {
                        pixels[i][cols] += isVer;
                        if (cols>0 && (pixels[i][cols-1] == isVer || pixels[i][cols-1] == (isHor+isVer))) {
                            countDouble++;
                        }
                        else if (cols > 0) {
                            pixels[i][cols-1] += isVer;
                        }
                        if (cols<(width-1) && (pixels[i][cols+1] == isVer || pixels[i][cols+1] == (isHor+isVer))) {
                            countDouble++;
                        }
                        else if (cols < width-1) {
                            pixels[i][cols+1] += isVer;
                        }
                    }
                }
            }
        }
        return (countDouble < isBoublePixels);
    }
    
    /**Check whether a pixel is at edge of the bounding box
     */
    private boolean isEdge(int rows, int cols) {
        return (rows == 0 || rows == (height -1) || cols == 0 || cols == (width - 1));
    }
    
    /** Create the 0,1 map representation of the region*/
    private void getDisplay() {
        /* initial a white map image*/
        pixels = new int [height][width];
        for (int rows = 0; rows < height; rows++) {
            for (int cols = 0; cols < width; cols++) {
                pixels[rows][cols] = 1;
                
            }
        }
        /* put the black pixels in the map*/
        int tempx = 0, tempy = 0;
        for (int i = 0; i < regionx.size(); i++) {
            tempx = ((Integer)regionx.get(i)).intValue();
            tempy = ((Integer)regiony.get(i)).intValue();
            
            //System.out.println((tempx-minX) + " " + (tempy-minY));
            
            /* set the pixel to black*/
            pixels[tempy-minY][tempx-minX] = 0;
        }
    }
    
    /**Override to show features' info
     */
    public String toString() {
        String info = "\n Width: " + width + "; Height: " + height;
        info += "\n Holes: " + holes + " (Up: " + upholes +
        "; Down: " + downholes
        + "; Center: " + centerholes +")";
        info += "\n Notches: " + notchs;
        info += "\n Horizental lines: " + hlongX + "; Vertical lines: "+ hlongY;
        
        info += "\n Area/BoxArea Ratio: " + abRatio;
        info += "\n Width/Height Ratio: " + bbRatio;
        info += "\n HolesArea/BoxArea Ratio: " + hbRatio;
        info += "\n NotchArea/BoxArea: " + nbRatio;
        
        getDisplay();
        for (int rows = 0; rows < height; rows++) {
            info += "\n";
            for (int cols = 0; cols < width; cols++) {
                info += " " + pixels[rows][cols];
            }
        }
        info += "\n";
        return info;
    }
    
    /**
     * Get the 0,1 map representation of the digit
     * return the pixel map
     */
    public int[][] getDigMap() {
        getDisplay();
        return pixels;
    }
    
    /**Return an double array for Extract features' value
     */
    public double[] toFeature() {
        
        double[] info = new double[13];
        info[0] = width;
        info[1] = height;
        info[2] = abRatio;  // Aera/boxAera
        info[3] = bbRatio;  //width/height
        info[4] = hbRatio;  //holeA/boxA
        info[5] = nbRatio;  //notchA/boxA
        info[6] = holes;   //holes
        info[7] = upholes;  //top holes
        info[8] = downholes; //downholes
        info[9] = centerholes;  //center holes
        info[10] = notchs;
        info[11] = hlongX;  //long H lines
        info[12] = hlongY;  //long V lines
        
        return info;
    }
    
    /** Print out the processed image*/
    private void PrintImage(int[][] pixelVal) {
        BufferedImage disImage = new BufferedImage(pixelVal[0].length, pixelVal.length,
                                        BufferedImage.TYPE_INT_RGB);
        for (int r = 0; r < pixelVal.length; r++) {
            for (int c = 0; c < pixelVal[0].length; c++) {
                disImage.setRGB(c, r, pixelVal[r][c]);
            }
        }
        
        disWindow = new JFrame();
        disImage.flush();
        JLabel lblImage = new JLabel(new ImageIcon(disImage), JLabel.CENTER);
        JButton btnDispose = new JButton("Close Window");
        
        Container windowContentPane = disWindow.getContentPane();
        windowContentPane.setLayout(new BoxLayout(windowContentPane, BoxLayout.Y_AXIS));
        windowContentPane.add(lblImage);
        windowContentPane.add(Box.createVerticalGlue());
        windowContentPane.add(btnDispose);
        
        btnDispose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                disWindow.dispose();
            }
        });
        disWindow.setResizable(false);
        disWindow.setLocation(100, 100);
        disWindow.pack();
        disWindow.show();
        
    }

}
