/*
 * FileDisplayPanel.java
 *
 * Created on May 12, 2004, 10:56 AM
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.File;
import javax.swing.table.*;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.util.Vector;
import java.net.URL;

/**
 *
 * @author  xiao
 */
public class FileDisplayPanel extends javax.swing.JPanel implements ActionListener{
    
    private JTable FileTable;
    private JScrollPane FileScrollPane;
    
    private TableModel dataModel;
    private Vector img_files = new Vector();
    final String[] names = {"#", "File Name"};
    final String[] items = {"File", "Directory"};
    String sep = String.valueOf(System.getProperty("file.separator").toCharArray()[0]);
    String current_dir = System.getProperty("user.dir") + sep;
    
    
    /** Creates new form FileDisplayPanel */
    public FileDisplayPanel() {
        
        setLayout(new java.awt.BorderLayout());
        setPreferredSize(new Dimension(260, 300));
        
        dataModel = new AbstractTableModel() {
            public int getColumnCount() {return names.length;};
            public int getRowCount() {return img_files.size();}
            public Object getValueAt(int row, int col) {
                if (col == 0) {
                    return new Integer(row);
                }
                else if (col==1) {
                    Object object = img_files.get(row);
                    if (object instanceof File) {
                        return ((File)object).getName();
                    }
                    else if (object instanceof URL) {
                        return ((URL)object).getFile();
                    }
                }
                return null;
            }
            public String getColumnName(int col) { return names[col];}
            public Class getColumnClass(int c) {
                return getValueAt(0, c).getClass();
            }
            public boolean isCellEditable(int row, int col) {
                return false;
            }
            public void setValueAt(Object aValue, int row, int col) {
            }
        };
        
        FileTable = new JTable(dataModel);
        TableColumn col = FileTable.getColumn("#");
        col.setMaxWidth(20);
        FileTable.sizeColumnsToFit(0);
        
        FileScrollPane = new JScrollPane(FileTable);
        EmptyBorder eb = new EmptyBorder(5, 5, 2, 5);
        FileScrollPane.setBorder(new CompoundBorder(eb, new EtchedBorder()));
        add(FileScrollPane);
        
        JPanel InsidePanel = new JPanel();
        JMenuBar InsideMenuBar = new JMenuBar();
        InsideMenuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        JMenu InsideMenu = (JMenu)InsideMenuBar.add(new JMenu("Add"));
        
        for (int i  = 0; i < items.length; i++) {
            JMenuItem item = InsideMenu.add(new JMenuItem(items[i]));
            item.addActionListener(this);
        }
        InsidePanel.add(InsideMenuBar);
        
        InsideMenuBar = new JMenuBar();
        InsideMenuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        InsideMenu = (JMenu) InsideMenuBar.add(new JMenu("Remove"));
        JMenuItem item = InsideMenu.add(new JMenuItem("Selected"));
        item.addActionListener(this);
        item = InsideMenu.add(new JMenuItem("All"));
        item.addActionListener(this);
        
        InsidePanel.add(InsideMenuBar);
        
        add("South", InsidePanel);
        
    }
    
    public void actionPerformed(ActionEvent e) {
        Object object = e.getSource();
        
        if (object instanceof JMenuItem) {
            JMenuItem mi = (JMenuItem)object;
            
            if (mi.getText().equals("File")) {
                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory(new File(current_dir));
 
                chooser.addChoosableFileFilter(new ImageFileFilter());
                chooser.setDialogTitle("Open Image Dialog");
                
                try {
                    int result = chooser.showOpenDialog(null);
                    
                    if (result == JFileChooser.APPROVE_OPTION) {
                        File f = chooser.getSelectedFile();
                        String select_file = f.getAbsolutePath();
                        current_dir = select_file.substring(0, select_file.lastIndexOf(sep)+1);
                        
                        loadFiles(select_file);
                        tableChanged();
                        
                        System.out.println("Current Dir: " + current_dir);
                    }
                }
                catch (Exception ex) {
                    JOptionPane.showMessageDialog((Component)null,
                    "Error Reading File:" + ex.toString(),
                    "File Open Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
            else if (mi.getText().equals("Directory")) {
                
                JFileChooser chooser = new JFileChooser();
                chooser.setCurrentDirectory(new File(current_dir));
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                chooser.setDialogTitle("Open Directory Dialog");
                
                try {
                    int result = chooser.showOpenDialog(null);
                    
                    if (result == JFileChooser.APPROVE_OPTION) {
                        File f = chooser.getSelectedFile();
                        current_dir = f.getAbsolutePath() + sep;
                        System.out.println("Current Dir:" + current_dir);
                        loadFiles(current_dir);
                        tableChanged();
                    }
                }
                catch (Exception ex) {
                    JOptionPane.showMessageDialog((Component)null,
                    "Error while trying to open:" + ex.toString(),
                    "Directory Open Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
            else if (mi.getText().equals("Selected")) {
                int rows[] = FileTable.getSelectedRows();
                Vector tmp = new Vector();
                for (int i = 0; i < rows.length; i++) {
                    tmp.add(img_files.get(rows[i]));
                }
                img_files.removeAll(tmp);
                tableChanged();
            }
            else if (mi.getText().equals("All")) {
                img_files.clear();
                tableChanged();
            }
        }
    }
    
    public void tableChanged() {
        FileTable.tableChanged(new TableModelEvent(dataModel));
    }
    
    public void loadFiles(String name) {
        try {
            File file  = new File(name);
            if (file != null && file.isDirectory()) {
                String files[] = file.list();
                for (int i = 0; i < files.length; i++) {
                    File leaf_file = new File(file.getAbsolutePath(), files[i]);
                    if (leaf_file.isDirectory()) {
                        loadFiles(leaf_file.getAbsolutePath());
                    }
                    else {
                        addFile(leaf_file);
                    }
                }
            }
            else if (file != null && file.exists()) {
                addFile(file);
            }
        }
        catch (SecurityException ex) {
        }
        catch (Exception ex){
        }
    }
    
    public void addFile(File file) {
        String s = file.getName();
        if ((s.endsWith(".gif")) || (s.endsWith(".jpg"))) {
            img_files.add(file);           
        }
    }
    
    public File getCurrentSelectedFile() {
        
        //System.out.println("Stack Size: " + img_files.size() +
        //" Selected Num: " + FileTable.getSelectedRowCount());
        if (img_files.isEmpty()|| FileTable.getSelectedRowCount()==0) {
            return null;
        }
        else {
            return (File)img_files.get(FileTable.getSelectedRow());            
        }
    }
    
    public static void main(String args[]) {
        
        FileDisplayPanel testPanel = new FileDisplayPanel();
        JFrame testFrame = new JFrame("File Open Panel Test");
        testFrame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });
        testFrame.getContentPane().add("Center", testPanel);
        testFrame.pack();
        
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 150;
        int h = 200;
        
        testFrame.setLocation(d.width/2 - w/2, d.height/2 - h/2);
        testFrame.setSize(w, h);
        testFrame.show();
    }
    
    
    
}
